#!/usr/bin/env node

import { Command } from 'commander';
import inquirer from 'inquirer';
import fs from 'fs-extra';
import path from 'path';
import chalk from 'chalk';
import { pascalCase, camelCase, kebabCase } from 'change-case';
import { fileURLToPath } from 'url';
import { dirname } from 'path';
import { exec } from 'child_process';
import { promisify } from 'util';

const execAsync = promisify(exec);
const __filename = fileURLToPath(import.meta.url);
const __dirname = dirname(__filename);

// Read package.json
const packageJson = JSON.parse(await fs.readFile(new URL('../package.json', import.meta.url)));

const program = new Command();

program
  .version(packageJson.version)
  .description('NestJS CRUD generator with Repository Pattern');

program
  .command('generate')
  .alias('g')
  .description('Generate CRUD components')
  .action(async () => {
    try {
      const answers = await inquirer.prompt([
        {
          type: 'input',
          name: 'name',
          message: 'Enter the resource name:',
          validate: input => input.length > 0
        },
        {
          type: 'list',
          name: 'orm',
          message: 'Which ORM would you like to use?',
          choices: [
            { name: 'TypeORM - Most popular ORM for TypeScript/Node.js', value: 'typeorm' },
            { name: 'Prisma - Modern DB toolkit with type safety', value: 'prisma' },
            { name: 'Mongoose - Elegant MongoDB ODM', value: 'mongoose' },
            { name: 'Sequelize - Feature-rich ORM for SQL databases', value: 'sequelize' },
            { name: 'MikroORM - TypeScript ORM with Identity Map pattern', value: 'mikroorm' }
          ]
        }
      ]);

      const { name, orm } = answers;

      // Get dependencies from package.json
      const commonDeps = Object.entries(packageJson.optionalDependencies || {})
        .map(([name, version]) => `${name}@${version}`);
      
      const ormDeps = Object.entries(packageJson.orm[orm].dependencies || {})
        .map(([name, version]) => `${name}@${version}`);
        if (orm === 'typeorm') {
          ormDeps.push('@types/node');
        }
      // Display dependencies information
      console.log(chalk.yellow('\nRequired dependencies:'));
      console.log(chalk.cyan('\nCommon dependencies:'));
      console.log(chalk.white(commonDeps.join('\n')));
      
      console.log(chalk.cyan('\nORM-specific dependencies:'));
      console.log(chalk.white(ormDeps.join('\n')));

      // Install dependencies automatically
      console.log(chalk.yellow('\nInstalling required dependencies...\n'));
      
      const dependencies = [...commonDeps, ...ormDeps];
      const currentDir = process.cwd();

      // Check if dependencies are already installed
      const nodeModulesPath = path.join(process.cwd(), 'node_modules');
      const checkDependency = (dep) => {
        const [name] = dep.split('@');
        return fs.existsSync(path.join(nodeModulesPath, name));
      };

      const missingDeps = dependencies.filter(dep => !checkDependency(dep));

      if (missingDeps.length === 0) {
        console.log(chalk.green('\n✓ All dependencies are already installed!\n'));
      } else {
        // Install dependencies automatically
        console.log(chalk.yellow('\nInstalling missing dependencies...\n'));
        console.log(chalk.white('Missing packages:', missingDeps.join(', ')));
        
        const installCommand = `cd "${currentDir}" && npm install --legacy-peer-deps ${missingDeps.join(' ')}`;
        
        try {
          const { stdout, stderr } = await execAsync(installCommand, { shell: true });
          if (stdout) console.log(chalk.green(stdout));
          if (stderr) console.log(chalk.yellow(stderr));
          console.log(chalk.green('\n✓ Dependencies installed successfully!\n'));
        } catch (error) {
          console.error(chalk.red('\n✗ Error installing dependencies:'), error.message);
          console.log(chalk.yellow('\nYou can install dependencies manually using:'));
          console.log(chalk.cyan('npm install --legacy-peer-deps'), chalk.white(missingDeps.join(' ')));
          console.log(chalk.yellow('\nOr using yarn:'));
          console.log(chalk.cyan('yarn add --ignore-peer-dependencies'), chalk.white(missingDeps.join(' ')));
          
          // Ask if user wants to continue without installing dependencies
          const { continueWithoutDeps } = await inquirer.prompt([
            {
              type: 'confirm',
              name: 'continueWithoutDeps',
              message: 'Do you want to continue without installing dependencies?',
              default: false
            }
          ]);

          if (!continueWithoutDeps) {
            console.log(chalk.red('\nAborted: Dependencies installation failed.'));
            process.exit(1);
          }
        }
      }

      // ORM-specific configurations
      const ormConfigs = {
        typeorm: {
          import: `import { TypeOrmModule } from '@nestjs/typeorm';\nimport { ${pascalCase(name)} } from './entities/${kebabCase(name)}.entity';`,
          imports: `TypeOrmModule.forFeature([${pascalCase(name)}])`,
        },
        prisma: {
          import: `import { PrismaModule } from '../prisma/prisma.module';`,
          imports: 'PrismaModule',
        },
        mongoose: {
          import: `import { MongooseModule } from '@nestjs/mongoose';\nimport { ${pascalCase(name)}, ${pascalCase(name)}Schema } from './entities/${kebabCase(name)}.entity';`,
          imports: `MongooseModule.forFeature([{ name: ${pascalCase(name)}.name, schema: ${pascalCase(name)}Schema }])`,
        },
        sequelize: {
          import: `import { SequelizeModule } from '@nestjs/sequelize';\nimport { ${pascalCase(name)} } from './entities/${kebabCase(name)}.entity';`,
          imports: `SequelizeModule.forFeature([${pascalCase(name)}])`,
        },
        mikroorm: {
          import: `import { MikroOrmModule } from '@mikro-orm/nestjs';\nimport { ${pascalCase(name)} } from './entities/${kebabCase(name)}.entity';`,
          imports: `MikroOrmModule.forFeature([${pascalCase(name)}])`,
        },
      };

      // Component generation
      const components = ['controller', 'service', 'repository', 'module', 'entity', 'dto'];
      const templatesDir = path.join(dirname(__dirname), 'templates');

      console.log(chalk.blue('\nGenerating CRUD components...\n'));

      for (const component of components) {
        try {
          const targetDir = path.join(process.cwd(), `src/${kebabCase(name)}`);
          let templatePath, targetPath, updateDtoPath;

          // Determine paths
          if (component === 'entity' || component === 'repository') {
            templatePath = path.join(templatesDir, 'repositories', orm, `${component}.template.ts`);
          } else {
            templatePath = path.join(templatesDir, 'common', `${component}.template.ts`);
          }

          if (component === 'dto') {
            await fs.ensureDir(path.join(targetDir, 'dto'));
            targetPath = path.join(targetDir, 'dto', `create-${kebabCase(name)}.dto.ts`);
            updateDtoPath = path.join(targetDir, 'dto', `update-${kebabCase(name)}.dto.ts`);
          } else if (component === 'entity') {
            await fs.ensureDir(path.join(targetDir, 'entities'));
            targetPath = path.join(targetDir, 'entities', `${kebabCase(name)}.entity.ts`);
          } else {
            targetPath = path.join(targetDir, `${kebabCase(name)}.${component}.ts`);
          }

          // Ensure directory exists
          await fs.ensureDir(path.dirname(targetPath));

          // Process template
          let templateContent = await fs.readFile(templatePath, 'utf8');
          templateContent = templateContent
            .replace(/{{pascalCase name}}/g, pascalCase(name))
            .replace(/{{camelCase name}}/g, camelCase(name))
            .replace(/{{kebabCase name}}/g, kebabCase(name));

          if (component === 'module') {
            const ormConfig = ormConfigs[orm];
            templateContent = templateContent
              .replace('{{importBlock}}', ormConfig.import)
              .replace('{{importsBlock}}', ormConfig.imports);
          }

          // Write files
          await fs.writeFile(targetPath, templateContent);
          console.log(chalk.green(`✓ Created ${component}: ${targetPath}`));

          if (component === 'dto') {
            const updateDtoTemplatePath = path.join(templatesDir, 'common', 'update.dto.template.ts');
            let updateDtoContent = await fs.readFile(updateDtoTemplatePath, 'utf8');
            
            updateDtoContent = updateDtoContent
              .replace(/{{pascalCase name}}/g, pascalCase(name))
              .replace(/{{camelCase name}}/g, camelCase(name))
              .replace(/{{kebabCase name}}/g, kebabCase(name));
            
            await fs.writeFile(updateDtoPath, updateDtoContent);
            console.log(chalk.green(`✓ Created update DTO: ${updateDtoPath}`));
          }

        } catch (error) {
          console.error(chalk.red(`✗ Error creating ${component}:`), error.message);
          if (error.code === 'ENOENT') {
            console.error(chalk.yellow(`  Make sure all template files exist in the correct location: ${templatePath}`));
          }
          throw error;
        }
      }

      console.log(chalk.green('\n✨ CRUD generation completed successfully!\n'));
      
    } catch (error) {
      console.error(chalk.red('\n✗ An error occurred during generation:'), error);
      process.exit(1);
    }
  });

program.parse(process.argv);